@@ -14,27 +14,35 @@ See the License for the specific language governing permissions and
 limitations under the License.
 */
 
+//go:generate go run azure_instance_types/gen.go
+
 package azure
 
 import (
+	"encoding/json"
 	"fmt"
 	"io"
+	"io/ioutil"
 	"os"
 	"strconv"
 	"strings"
 	"time"
 
+	"github.com/Azure/go-autorest/autorest"
+	"github.com/Azure/go-autorest/autorest/adal"
 	"github.com/Azure/go-autorest/autorest/azure"
-	"gopkg.in/gcfg.v1"
+
 	"k8s.io/autoscaler/cluster-autoscaler/cloudprovider"
 	"k8s.io/autoscaler/cluster-autoscaler/config/dynamic"
 	"k8s.io/klog"
+	providerazure "k8s.io/legacy-cloud-providers/azure"
+	azclients "k8s.io/legacy-cloud-providers/azure/clients"
+	"k8s.io/legacy-cloud-providers/azure/retry"
 )
 
 const (
 	vmTypeVMSS     = "vmss"
 	vmTypeStandard = "standard"
-	vmTypeACS      = "acs"
 	vmTypeAKS      = "aks"
 
 	scaleToZeroSupportedStandard = false
@@ -49,6 +57,17 @@ const (
 	autoDiscovererTypeLabel        = "label"
 	labelAutoDiscovererKeyMinNodes = "min"
 	labelAutoDiscovererKeyMaxNodes = "max"
+	metadataURL                    = "http://169.254.169.254/metadata/instance"
+
+	// backoff
+	backoffRetriesDefault  = 6
+	backoffExponentDefault = 1.5
+	backoffDurationDefault = 5 // in seconds
+	backoffJitterDefault   = 1.0
+
+	// rate limit
+	rateLimitQPSDefault    = 1.0
+	rateLimitBucketDefault = 5
 )
 
 var validLabelAutoDiscovererKeys = strings.Join([]string{
@@ -60,10 +79,6 @@ var validLabelAutoDiscovererKeys = strings.Join([]string{
 type labelAutoDiscoveryConfig struct {
 	// Key-values to match on.
 	Selector map[string]string
-	// MinSize specifies the minimum size for all MIGs that match Re.
-	MinSize int
-	// MaxSize specifies the maximum size for all MIGs that match Re.
-	MaxSize int
 }
 
 // AzureManager handles Azure communication and data caching.
@@ -78,9 +93,25 @@ type AzureManager struct {
 	explicitlyConfigured  map[string]bool
 }
 
+// CloudProviderRateLimitConfig indicates the rate limit config for each clients.
+type CloudProviderRateLimitConfig struct {
+	// The default rate limit config options.
+	azclients.RateLimitConfig
+
+	// Rate limit config for each clients. Values would override default settings above.
+	InterfaceRateLimit              *azclients.RateLimitConfig `json:"interfaceRateLimit,omitempty" yaml:"interfaceRateLimit,omitempty"`
+	VirtualMachineRateLimit         *azclients.RateLimitConfig `json:"virtualMachineRateLimit,omitempty" yaml:"virtualMachineRateLimit,omitempty"`
+	StorageAccountRateLimit         *azclients.RateLimitConfig `json:"storageAccountRateLimit,omitempty" yaml:"storageAccountRateLimit,omitempty"`
+	DiskRateLimit                   *azclients.RateLimitConfig `json:"diskRateLimit,omitempty" yaml:"diskRateLimit,omitempty"`
+	VirtualMachineScaleSetRateLimit *azclients.RateLimitConfig `json:"virtualMachineScaleSetRateLimit,omitempty" yaml:"virtualMachineScaleSetRateLimit,omitempty"`
+}
+
 // Config holds the configuration parsed from the --cloud-config flag
 type Config struct {
+	CloudProviderRateLimitConfig
+
 	Cloud          string `json:"cloud" yaml:"cloud"`
+	Location       string `json:"location" yaml:"location"`
 	TenantID       string `json:"tenantId" yaml:"tenantId"`
 	SubscriptionID string `json:"subscriptionId" yaml:"subscriptionId"`
 	ResourceGroup  string `json:"resourceGroup" yaml:"resourceGroup"`
@@ -91,46 +122,143 @@ type Config struct {
 	AADClientCertPath           string `json:"aadClientCertPath" yaml:"aadClientCertPath"`
 	AADClientCertPassword       string `json:"aadClientCertPassword" yaml:"aadClientCertPassword"`
 	UseManagedIdentityExtension bool   `json:"useManagedIdentityExtension" yaml:"useManagedIdentityExtension"`
+	UserAssignedIdentityID      string `json:"userAssignedIdentityID" yaml:"userAssignedIdentityID"`
 
 	// Configs only for standard vmType (agent pools).
 	Deployment           string                 `json:"deployment" yaml:"deployment"`
 	DeploymentParameters map[string]interface{} `json:"deploymentParameters" yaml:"deploymentParameters"`
 
-	//Configs only for ACS/AKS
+	//Configs only for AKS
 	ClusterName string `json:"clusterName" yaml:"clusterName"`
 	//Config only for AKS
 	NodeResourceGroup string `json:"nodeResourceGroup" yaml:"nodeResourceGroup"`
+
+	// VMSS metadata cache TTL in seconds, only applies for vmss type
+	VmssCacheTTL int64 `json:"vmssCacheTTL" yaml:"vmssCacheTTL"`
+
+	// number of latest deployments that will not be deleted
+	MaxDeploymentsCount int64 `json:"maxDeploymentsCount" yaml:"maxDeploymentsCount"`
+
+	// Enable exponential backoff to manage resource request retries
+	CloudProviderBackoff         bool    `json:"cloudProviderBackoff,omitempty" yaml:"cloudProviderBackoff,omitempty"`
+	CloudProviderBackoffRetries  int     `json:"cloudProviderBackoffRetries,omitempty" yaml:"cloudProviderBackoffRetries,omitempty"`
+	CloudProviderBackoffExponent float64 `json:"cloudProviderBackoffExponent,omitempty" yaml:"cloudProviderBackoffExponent,omitempty"`
+	CloudProviderBackoffDuration int     `json:"cloudProviderBackoffDuration,omitempty" yaml:"cloudProviderBackoffDuration,omitempty"`
+	CloudProviderBackoffJitter   float64 `json:"cloudProviderBackoffJitter,omitempty" yaml:"cloudProviderBackoffJitter,omitempty"`
+}
+
+// InitializeCloudProviderRateLimitConfig initializes rate limit configs.
+func InitializeCloudProviderRateLimitConfig(config *CloudProviderRateLimitConfig) {
+	if config == nil {
+		return
+	}
+
+	// Assign read rate limit defaults if no configuration was passed in.
+	if config.CloudProviderRateLimitQPS == 0 {
+		config.CloudProviderRateLimitQPS = rateLimitQPSDefault
+	}
+	if config.CloudProviderRateLimitBucket == 0 {
+		config.CloudProviderRateLimitBucket = rateLimitBucketDefault
+	}
+	// Assing write rate limit defaults if no configuration was passed in.
+	if config.CloudProviderRateLimitQPSWrite == 0 {
+		config.CloudProviderRateLimitQPSWrite = config.CloudProviderRateLimitQPS
+	}
+	if config.CloudProviderRateLimitBucketWrite == 0 {
+		config.CloudProviderRateLimitBucketWrite = config.CloudProviderRateLimitBucket
+	}
+
+	config.InterfaceRateLimit = overrideDefaultRateLimitConfig(&config.RateLimitConfig, config.InterfaceRateLimit)
+	config.VirtualMachineRateLimit = overrideDefaultRateLimitConfig(&config.RateLimitConfig, config.VirtualMachineRateLimit)
+	config.StorageAccountRateLimit = overrideDefaultRateLimitConfig(&config.RateLimitConfig, config.StorageAccountRateLimit)
+	config.DiskRateLimit = overrideDefaultRateLimitConfig(&config.RateLimitConfig, config.DiskRateLimit)
+	config.VirtualMachineScaleSetRateLimit = overrideDefaultRateLimitConfig(&config.RateLimitConfig, config.VirtualMachineScaleSetRateLimit)
+}
+
+// overrideDefaultRateLimitConfig overrides the default CloudProviderRateLimitConfig.
+func overrideDefaultRateLimitConfig(defaults, config *azclients.RateLimitConfig) *azclients.RateLimitConfig {
+	// If config not set, apply defaults.
+	if config == nil {
+		return defaults
+	}
+
+	// Remain disabled if it's set explicitly.
+	if !config.CloudProviderRateLimit {
+		return &azclients.RateLimitConfig{CloudProviderRateLimit: false}
+	}
+
+	// Apply default values.
+	if config.CloudProviderRateLimitQPS == 0 {
+		config.CloudProviderRateLimitQPS = defaults.CloudProviderRateLimitQPS
+	}
+	if config.CloudProviderRateLimitBucket == 0 {
+		config.CloudProviderRateLimitBucket = defaults.CloudProviderRateLimitBucket
+	}
+	if config.CloudProviderRateLimitQPSWrite == 0 {
+		config.CloudProviderRateLimitQPSWrite = defaults.CloudProviderRateLimitQPSWrite
+	}
+	if config.CloudProviderRateLimitBucketWrite == 0 {
+		config.CloudProviderRateLimitBucketWrite = defaults.CloudProviderRateLimitBucketWrite
+	}
+
+	return config
+}
+
+func (cfg *Config) getAzureClientConfig(servicePrincipalToken *adal.ServicePrincipalToken, env *azure.Environment) *azclients.ClientConfig {
+	azClientConfig := &azclients.ClientConfig{
+		Location:                cfg.Location,
+		SubscriptionID:          cfg.SubscriptionID,
+		ResourceManagerEndpoint: env.ResourceManagerEndpoint,
+		Authorizer:              autorest.NewBearerAuthorizer(servicePrincipalToken),
+		Backoff:                 &retry.Backoff{Steps: 1},
+	}
+
+	if cfg.CloudProviderBackoff {
+		azClientConfig.Backoff = &retry.Backoff{
+			Steps:    cfg.CloudProviderBackoffRetries,
+			Factor:   cfg.CloudProviderBackoffExponent,
+			Duration: time.Duration(cfg.CloudProviderBackoffDuration) * time.Second,
+			Jitter:   cfg.CloudProviderBackoffJitter,
+		}
+	}
+
+	return azClientConfig
 }
 
 // TrimSpace removes all leading and trailing white spaces.
-func (c *Config) TrimSpace() {
-	c.Cloud = strings.TrimSpace(c.Cloud)
-	c.TenantID = strings.TrimSpace(c.TenantID)
-	c.SubscriptionID = strings.TrimSpace(c.SubscriptionID)
-	c.ResourceGroup = strings.TrimSpace(c.ResourceGroup)
-	c.VMType = strings.TrimSpace(c.VMType)
-	c.AADClientID = strings.TrimSpace(c.AADClientID)
-	c.AADClientSecret = strings.TrimSpace(c.AADClientSecret)
-	c.AADClientCertPath = strings.TrimSpace(c.AADClientCertPath)
-	c.AADClientCertPassword = strings.TrimSpace(c.AADClientCertPassword)
-	c.Deployment = strings.TrimSpace(c.Deployment)
-	c.ClusterName = strings.TrimSpace(c.ClusterName)
-	c.NodeResourceGroup = strings.TrimSpace(c.NodeResourceGroup)
+func (cfg *Config) TrimSpace() {
+	cfg.Cloud = strings.TrimSpace(cfg.Cloud)
+	cfg.Location = strings.TrimSpace(cfg.Location)
+	cfg.TenantID = strings.TrimSpace(cfg.TenantID)
+	cfg.SubscriptionID = strings.TrimSpace(cfg.SubscriptionID)
+	cfg.ResourceGroup = strings.TrimSpace(cfg.ResourceGroup)
+	cfg.VMType = strings.TrimSpace(cfg.VMType)
+	cfg.AADClientID = strings.TrimSpace(cfg.AADClientID)
+	cfg.AADClientSecret = strings.TrimSpace(cfg.AADClientSecret)
+	cfg.AADClientCertPath = strings.TrimSpace(cfg.AADClientCertPath)
+	cfg.AADClientCertPassword = strings.TrimSpace(cfg.AADClientCertPassword)
+	cfg.Deployment = strings.TrimSpace(cfg.Deployment)
+	cfg.ClusterName = strings.TrimSpace(cfg.ClusterName)
+	cfg.NodeResourceGroup = strings.TrimSpace(cfg.NodeResourceGroup)
 }
 
 // CreateAzureManager creates Azure Manager object to work with Azure.
 func CreateAzureManager(configReader io.Reader, discoveryOpts cloudprovider.NodeGroupDiscoveryOptions) (*AzureManager, error) {
 	var err error
-	var cfg Config
+	cfg := &Config{}
 
 	if configReader != nil {
-		if err := gcfg.ReadInto(&cfg, configReader); err != nil {
-			klog.Errorf("Couldn't read config: %v", err)
-			return nil, err
+		body, err := ioutil.ReadAll(configReader)
+		if err != nil {
+			return nil, fmt.Errorf("failed to read config: %v", err)
+		}
+		err = json.Unmarshal(body, cfg)
+		if err != nil {
+			return nil, fmt.Errorf("failed to unmarshal config body: %v", err)
 		}
 	} else {
 		cfg.Cloud = os.Getenv("ARM_CLOUD")
-		cfg.SubscriptionID = os.Getenv("ARM_SUBSCRIPTION_ID")
+		cfg.Location = os.Getenv("LOCATION")
 		cfg.ResourceGroup = os.Getenv("ARM_RESOURCE_GROUP")
 		cfg.TenantID = os.Getenv("ARM_TENANT_ID")
 		cfg.AADClientID = os.Getenv("ARM_CLIENT_ID")
@@ -142,6 +270,12 @@ func CreateAzureManager(configReader io.Reader, discoveryOpts cloudprovider.Node
 		cfg.ClusterName = os.Getenv("AZURE_CLUSTER_NAME")
 		cfg.NodeResourceGroup = os.Getenv("AZURE_NODE_RESOURCE_GROUP")
 
+		subscriptionID, err := getSubscriptionIdFromInstanceMetadata()
+		if err != nil {
+			return nil, err
+		}
+		cfg.SubscriptionID = subscriptionID
+
 		useManagedIdentityExtensionFromEnv := os.Getenv("ARM_USE_MANAGED_IDENTITY_EXTENSION")
 		if len(useManagedIdentityExtensionFromEnv) > 0 {
 			cfg.UseManagedIdentityExtension, err = strconv.ParseBool(useManagedIdentityExtensionFromEnv)
@@ -149,9 +283,83 @@ func CreateAzureManager(configReader io.Reader, discoveryOpts cloudprovider.Node
 				return nil, err
 			}
 		}
+
+		userAssignedIdentityIDFromEnv := os.Getenv("ARM_USER_ASSIGNED_IDENTITY_ID")
+		if userAssignedIdentityIDFromEnv != "" {
+			cfg.UserAssignedIdentityID = userAssignedIdentityIDFromEnv
+		}
+
+		if vmssCacheTTL := os.Getenv("AZURE_VMSS_CACHE_TTL"); vmssCacheTTL != "" {
+			cfg.VmssCacheTTL, err = strconv.ParseInt(vmssCacheTTL, 10, 0)
+			if err != nil {
+				return nil, fmt.Errorf("failed to parse AZURE_VMSS_CACHE_TTL %q: %v", vmssCacheTTL, err)
+			}
+		}
+
+		if threshold := os.Getenv("AZURE_MAX_DEPLOYMENT_COUNT"); threshold != "" {
+			cfg.MaxDeploymentsCount, err = strconv.ParseInt(threshold, 10, 0)
+			if err != nil {
+				return nil, fmt.Errorf("failed to parse AZURE_MAX_DEPLOYMENT_COUNT %q: %v", threshold, err)
+			}
+		}
+
+		if enableBackoff := os.Getenv("ENABLE_BACKOFF"); enableBackoff != "" {
+			cfg.CloudProviderBackoff, err = strconv.ParseBool(enableBackoff)
+			if err != nil {
+				return nil, fmt.Errorf("failed to parse ENABLE_BACKOFF %q: %v", enableBackoff, err)
+			}
+		}
+
+		if cfg.CloudProviderBackoff {
+			if backoffRetries := os.Getenv("BACKOFF_RETRIES"); backoffRetries != "" {
+				retries, err := strconv.ParseInt(backoffRetries, 10, 0)
+				if err != nil {
+					return nil, fmt.Errorf("failed to parse BACKOFF_RETRIES %q: %v", retries, err)
+				}
+				cfg.CloudProviderBackoffRetries = int(retries)
+			} else {
+				cfg.CloudProviderBackoffRetries = backoffRetriesDefault
+			}
+
+			if backoffExponent := os.Getenv("BACKOFF_EXPONENT"); backoffExponent != "" {
+				cfg.CloudProviderBackoffExponent, err = strconv.ParseFloat(backoffExponent, 64)
+				if err != nil {
+					return nil, fmt.Errorf("failed to parse BACKOFF_EXPONENT %q: %v", backoffExponent, err)
+				}
+			} else {
+				cfg.CloudProviderBackoffExponent = backoffExponentDefault
+			}
+
+			if backoffDuration := os.Getenv("BACKOFF_DURATION"); backoffDuration != "" {
+				duration, err := strconv.ParseInt(backoffDuration, 10, 0)
+				if err != nil {
+					return nil, fmt.Errorf("failed to parse BACKOFF_DURATION %q: %v", backoffDuration, err)
+				}
+				cfg.CloudProviderBackoffDuration = int(duration)
+			} else {
+				cfg.CloudProviderBackoffDuration = backoffDurationDefault
+			}
+
+			if backoffJitter := os.Getenv("BACKOFF_JITTER"); backoffJitter != "" {
+				cfg.CloudProviderBackoffJitter, err = strconv.ParseFloat(backoffJitter, 64)
+				if err != nil {
+					return nil, fmt.Errorf("failed to parse BACKOFF_JITTER %q: %v", backoffJitter, err)
+				}
+			} else {
+				cfg.CloudProviderBackoffJitter = backoffJitterDefault
+			}
+		}
 	}
 	cfg.TrimSpace()
 
+	if cloudProviderRateLimit := os.Getenv("CLOUD_PROVIDER_RATE_LIMIT"); cloudProviderRateLimit != "" {
+		cfg.CloudProviderRateLimit, err = strconv.ParseBool(cloudProviderRateLimit)
+		if err != nil {
+			return nil, fmt.Errorf("failed to parse CLOUD_PROVIDER_RATE_LIMIT: %q, %v", cloudProviderRateLimit, err)
+		}
+	}
+	InitializeCloudProviderRateLimitConfig(&cfg.CloudProviderRateLimitConfig)
+
 	// Defaulting vmType to vmss.
 	if cfg.VMType == "" {
 		cfg.VMType = vmTypeVMSS
@@ -168,6 +376,10 @@ func CreateAzureManager(configReader io.Reader, discoveryOpts cloudprovider.Node
 		cfg.DeploymentParameters = parameters
 	}
 
+	if cfg.MaxDeploymentsCount == 0 {
+		cfg.MaxDeploymentsCount = int64(defaultMaxDeploymentsCount)
+	}
+
 	// Defaulting env to Azure Public Cloud.
 	env := azure.PublicCloud
 	if cfg.Cloud != "" {
@@ -177,20 +389,20 @@ func CreateAzureManager(configReader io.Reader, discoveryOpts cloudprovider.Node
 		}
 	}
 
-	if err := validateConfig(&cfg); err != nil {
+	if err := validateConfig(cfg); err != nil {
 		return nil, err
 	}
 
 	klog.Infof("Starting azure manager with subscription ID %q", cfg.SubscriptionID)
 
-	azClient, err := newAzClient(&cfg, &env)
+	azClient, err := newAzClient(cfg, &env)
 	if err != nil {
 		return nil, err
 	}
 
 	// Create azure manager.
 	manager := &AzureManager{
-		config:               &cfg,
+		config:               cfg,
 		env:                  env,
 		azClient:             azClient,
 		explicitlyConfigured: make(map[string]bool),
@@ -255,10 +467,8 @@ func (m *AzureManager) buildAsgFromSpec(spec string) (cloudprovider.NodeGroup, e
 		return NewAgentPool(s, m)
 	case vmTypeVMSS:
 		return NewScaleSet(s, m)
-	case vmTypeACS:
-		fallthrough
 	case vmTypeAKS:
-		return NewContainerServiceAgentPool(s, m)
+		return NewAKSAgentPool(s, m)
 	default:
 		return nil, fmt.Errorf("vmtype %s not supported", m.config.VMType)
 	}
@@ -274,8 +484,13 @@ func (m *AzureManager) Refresh() error {
 }
 
 func (m *AzureManager) forceRefresh() error {
+	// TODO: Refactor some of this logic out of forceRefresh and
+	// consider merging the list call with the Nodes() call
 	if err := m.fetchAutoAsgs(); err != nil {
 		klog.Errorf("Failed to fetch ASGs: %v", err)
+	}
+	if err := m.regenerateCache(); err != nil {
+		klog.Errorf("Failed to regenerate ASG cache: %v", err)
 		return err
 	}
 	m.lastRefresh = time.Now()
@@ -366,7 +581,6 @@ func (m *AzureManager) getFilteredAutoscalingGroups(filter []labelAutoDiscoveryC
 		asgs, err = m.listScaleSets(filter)
 	case vmTypeStandard:
 		asgs, err = m.listAgentPools(filter)
-	case vmTypeACS:
 	case vmTypeAKS:
 		return nil, nil
 	default:
@@ -380,16 +594,17 @@ func (m *AzureManager) getFilteredAutoscalingGroups(filter []labelAutoDiscoveryC
 }
 
 // listScaleSets gets a list of scale sets and instanceIDs.
-func (m *AzureManager) listScaleSets(filter []labelAutoDiscoveryConfig) (asgs []cloudprovider.NodeGroup, err error) {
+func (m *AzureManager) listScaleSets(filter []labelAutoDiscoveryConfig) ([]cloudprovider.NodeGroup, error) {
 	ctx, cancel := getContextWithCancel()
 	defer cancel()
 
-	result, err := m.azClient.virtualMachineScaleSetsClient.List(ctx, m.config.ResourceGroup)
-	if err != nil {
-		klog.Errorf("VirtualMachineScaleSetsClient.List for %v failed: %v", m.config.ResourceGroup, err)
-		return nil, err
+	result, rerr := m.azClient.virtualMachineScaleSetsClient.List(ctx, m.config.ResourceGroup)
+	if rerr != nil {
+		klog.Errorf("VirtualMachineScaleSetsClient.List for %v failed: %v", m.config.ResourceGroup, rerr)
+		return nil, rerr.Error()
 	}
 
+	var asgs []cloudprovider.NodeGroup
 	for _, scaleSet := range result {
 		if len(filter) > 0 {
 			if scaleSet.Tags == nil || len(scaleSet.Tags) == 0 {
@@ -414,7 +629,10 @@ func (m *AzureManager) listScaleSets(filter []labelAutoDiscoveryConfig) (asgs []
 				return asgs, fmt.Errorf("invalid minimum size specified for vmss: %s", err)
 			}
 		} else {
-			return asgs, fmt.Errorf("no minimum size specified for vmss: %s", err)
+			return asgs, fmt.Errorf("no minimum size specified for vmss: %s", *scaleSet.Name)
+		}
+		if spec.MinSize < 0 {
+			return asgs, fmt.Errorf("minimum size must be a non-negative number of nodes")
 		}
 		if val, ok := scaleSet.Tags["max"]; ok {
 			if maxSize, err := strconv.Atoi(*val); err == nil {
@@ -423,7 +641,7 @@ func (m *AzureManager) listScaleSets(filter []labelAutoDiscoveryConfig) (asgs []
 				return asgs, fmt.Errorf("invalid maximum size specified for vmss: %s", err)
 			}
 		} else {
-			return asgs, fmt.Errorf("no maximum size specified for vmss: %s", err)
+			return asgs, fmt.Errorf("no maximum size specified for vmss: %s", *scaleSet.Name)
 		}
 		if spec.MaxSize < 1 {
 			return asgs, fmt.Errorf("maximum size must be greater than 1 node")
@@ -488,8 +706,6 @@ func parseLabelAutoDiscoverySpecs(o cloudprovider.NodeGroupDiscoveryOptions) ([]
 func parseLabelAutoDiscoverySpec(spec string) (labelAutoDiscoveryConfig, error) {
 	cfg := labelAutoDiscoveryConfig{
 		Selector: make(map[string]string),
-		MinSize:  1,
-		MaxSize:  -1,
 	}
 
 	tokens := strings.Split(spec, ":")
@@ -506,31 +722,30 @@ func parseLabelAutoDiscoverySpec(spec string) (labelAutoDiscoveryConfig, error)
 		if len(kv) != 2 {
 			return cfg, fmt.Errorf("invalid key=value pair %s", kv)
 		}
-
 		k, v := kv[0], kv[1]
-
-		switch k {
-		case labelAutoDiscovererKeyMinNodes:
-			if minSize, err := strconv.Atoi(v); err == nil {
-				cfg.MinSize = minSize
-			} else {
-				return cfg, fmt.Errorf("invalid minimum nodes: %s", v)
-			}
-		case labelAutoDiscovererKeyMaxNodes:
-			if maxSize, err := strconv.Atoi(v); err == nil {
-				cfg.MaxSize = maxSize
-			} else {
-				return cfg, fmt.Errorf("invalid maximum nodes: %s", v)
-			}
-		default:
-			cfg.Selector[k] = v
+		if k == "" || v == "" {
+			return cfg, fmt.Errorf("empty value not allowed in key=value tag pairs")
 		}
-	}
-	if cfg.MaxSize < 1 {
-		return cfg, fmt.Errorf("maximum size must be greater than 1 node")
-	}
-	if cfg.MaxSize < cfg.MinSize {
-		return cfg, fmt.Errorf("maximum size must be greater than minimum size")
+		cfg.Selector[k] = v
 	}
 	return cfg, nil
 }
+
+// getSubscriptionId reads the Subscription ID from the instance metadata.
+func getSubscriptionIdFromInstanceMetadata() (string, error) {
+	subscriptionID, present := os.LookupEnv("ARM_SUBSCRIPTION_ID")
+	if !present {
+		metadataService, err := providerazure.NewInstanceMetadataService(metadataURL)
+		if err != nil {
+			return "", err
+		}
+
+		metadata, err := metadataService.GetMetadata(0)
+		if err != nil {
+			return "", err
+		}
+
+		return metadata.Compute.SubscriptionID, nil
+	}
+	return subscriptionID, nil
+}
