@@ -60,6 +60,7 @@ type AutoscalingGceClient interface {
 	FetchMigTemplate(GceRef) (*gce.InstanceTemplate, error)
 	FetchMigsWithName(zone string, filter *regexp.Regexp) ([]string, error)
 	FetchZones(region string) ([]string, error)
+	FetchAvailableCpuPlatforms() (map[string][]string, error)
 
 	// modifying resources
 	ResizeMig(GceRef, int64) error
@@ -116,16 +117,21 @@ func (client *autoscalingGceClientV1) FetchMachineType(zone, machineType string)
 
 func (client *autoscalingGceClientV1) FetchMachineTypes(zone string) ([]*gce.MachineType, error) {
 	registerRequest("machine_types", "list")
-	machines, err := client.gceService.MachineTypes.List(client.projectId, zone).Do()
+	var machineTypes []*gce.MachineType
+	err := client.gceService.MachineTypes.List(client.projectId, zone).Pages(
+		context.TODO(),
+		func(page *gce.MachineTypeList) error {
+			machineTypes = append(machineTypes, page.Items...)
+			return nil
+		})
 	if err != nil {
 		return nil, err
 	}
-	return machines.Items, nil
+	return machineTypes, nil
 }
 
 func (client *autoscalingGceClientV1) FetchAllMigs(zone string) ([]*gce.InstanceGroupManager, error) {
 	registerRequest("instance_group_managers", "list")
-
 	var migs []*gce.InstanceGroupManager
 	err := client.gceService.InstanceGroupManagers.List(client.projectId, zone).Pages(
 		context.TODO(),
@@ -312,6 +318,22 @@ func (client *autoscalingGceClientV1) FetchZones(region string) ([]string, error
 	return zones, nil
 }
 
+func (client *autoscalingGceClientV1) FetchAvailableCpuPlatforms() (map[string][]string, error) {
+	availableCpuPlatforms := make(map[string][]string)
+	err := client.gceService.Zones.List(client.projectId).Pages(
+		context.TODO(),
+		func(zones *gce.ZoneList) error {
+			for _, zone := range zones.Items {
+				availableCpuPlatforms[zone.Name] = zone.AvailableCpuPlatforms
+			}
+			return nil
+		})
+	if err != nil {
+		return nil, err
+	}
+	return availableCpuPlatforms, nil
+}
+
 func (client *autoscalingGceClientV1) FetchMigTemplate(migRef GceRef) (*gce.InstanceTemplate, error) {
 	registerRequest("instance_group_managers", "get")
 	igm, err := client.gceService.InstanceGroupManagers.Get(migRef.Project, migRef.Zone, migRef.Name).Do()
